/*
 * Common routines.
 *
 * Copyright (C) 2019, 2020, 2022 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include <stdio.h>
#include "types.h"

/* Error message generation. */

void open_error(const char *dir, const char *name);

/* Filename manipulation. */

char *make_basename(const char *path);
char *make_dirname(const char *path);

/* File opening. */

FILE *get_output_file(const char *filename_template, const char *dir, const char *name);

/* Interface attribute access. */

char *get_attribute_value(struct attribute *attr, const char *name);
char *get_opcode(struct attribute *attr);
char *get_opcode_type(struct attribute *attr);
char *get_protocol(struct attribute *attr);
char *get_operation_wrapper_prefix(struct attribute *attr);
int have_attribute(struct attribute *attr, const char *name);
int have_attribute_value(struct attribute *attr, const char *name, const char *value);
int have_identifier(struct identifier *ident, const char *value);

/* Function role selection. */

enum function_role get_server_function_role(struct signature *sig);

/* Dispatcher object conversion (from compound to individual interfaces). */

char *get_object_conversion(struct interface *iface, int compound);

/* Interface and operation naming. */

char *get_interface_class_name(struct interface *iface,
                               enum component_role component);

char *get_signature_operation_name(struct interface *iface,
                                   struct signature *sig,
                                   enum component_role component,
                                   enum signature_role role,
                                   enum language lang);

char *get_operation_name(struct interface *iface, struct signature *sig);
char *get_opcode_identifier(char *protocol, char *opname);

const char *get_parameter_decoration(struct parameter *param,
                                     enum parameter_role role,
                                     enum function_role function);

char *get_parameter_name(struct identifier *ident);
char *get_signature_terminator(enum signature_role role);

void count_parameters(struct parameter *param, int *input_words,
                      int *input_items, int *output_words, int *output_items);

int get_max_input_items(struct signature *sig);

/* Message access details. */

char *access_name(struct parameter *param, enum component_role component, int writing);
char *item_type_name(struct parameter *param, enum component_role component, int writing);
int specify_receive_fpage(struct parameter *param, enum component_role component, int writing);
char *reference_message(enum component_role component, enum function_role function);
char *structure_prefix(enum specifier direction);
int writing_to_message(enum component_role component, enum specifier direction);
